#include "Controller.h"

Controller::Controller(void) {
	SDLKey k[] = CONTROLLER_KEYS;
	nControllerKeys = sizeof(k) / sizeof(SDLKey);
	
	controllerKeys = new SDLKey[nControllerKeys];
	actions = new bool[nControllerKeys];

	for (int i = 0; i < nControllerKeys; i++) {
		actions[i] = false;
		controllerKeys[i] = k[i];
	}
}

Controller::~Controller(void) {
	delete[] controllerKeys;
	delete[] actions;
}

std::string Controller::getActionString(int action) {
	return actionsStr[action];
}

int Controller::getActionFromString(std::string& str) {
	for (int i = 0; i < nControllerKeys; i++) {
		if (str.compare(actionsStr[i]) == 0) {
			return i;
		}
	}
	return -1;
}

void Controller::loadControls(Json::Value& node) {
	unsigned int n = node.size();
	ControllerActions curAction;
	Json::Value::Members strActions = node.getMemberNames();

	for (unsigned int i = 0; i < n; i++) {
		curAction = (ControllerActions)getActionFromString(strActions[i]);
		
		controllerKeys[curAction] =
			(SDLKey)node.get(strActions[i], controllerKeys[curAction]).asInt();
	}

}

void Controller::pollEvent(SDL_Event* event) {
	if (event->type == SDL_KEYDOWN) {
		for (int i = 0; i < nControllerKeys; i++) {
			if (event->key.keysym.sym == controllerKeys[i]) {
				actions[i] = true;
			}
		}
	} else if (event->type == SDL_KEYUP) {
		for (int i = 0; i < nControllerKeys; i++) {
			if (event->key.keysym.sym == controllerKeys[i]) {
				actions[i] = false;
			}
		}
	}
}

bool Controller::isDown(ControllerActions action) {
	return actions[action];
}

bool Controller::isPressed(ControllerActions action) {
	bool ret = isDown(action);
	actions[action] = false;
	return ret;
}

void Controller::dumpJSON() {
	for (int i = 0; i < nControllerKeys; i++) {
		d("\"%s\": %d, // %s\n", getActionString(i).c_str(), controllerKeys[i], SDL_GetKeyName(controllerKeys[i]));
	}

}